package junit.logswingui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import junit.framework.Test;
import junit.framework.TestResult;
import junit.log4j.AppenderListener;
import junit.log4j.JUnitAppenderSingleton;
import junit.log4j.LoggedTestResult;
import junit.swingui.TestRunContext;

/**
 * A Swing based user interface to run tests and protocol their results
 * with <a href="http://jakarta.apache.org/log4j">Log4J</a>.
 * <p>
 *
 * <p>Enter the name of a class which either provides a static
 * suite method or is a subclass of TestCase.</p>
 * <pre>
 * Synopsis: java junit.logswingui.TestRunner [-noloading] [TestCase]
 * </pre>
 * <p>TestRunner takes as an optional argument the name of the testcase
 * class to be run.</p>
 */
public class TestRunner extends junit.swingui.TestRunner implements AppenderListener, TestRunContext {
    protected JButton fProtocolButton;
    protected TestProtocolView fProtocolView;
    protected LoggedTestResult fLoggedTestResult;
    protected DateFormat dateFormat;
    protected long startTime;

  
    public TestRunner() {
        JUnitAppenderSingleton.addListener(TestRunner.this);
        dateFormat = new SimpleDateFormat("EEEE d. MMMM yyyy, H:mm:ss (zzzz)");
    } 


    public static void main(String[] args) {
        new TestRunner().start(args);
    }


    protected void aboutToStart(final Test testSuite) {

        startTime= System.currentTimeMillis();
        writeProtocolHeader(new Date(startTime));

        super.aboutToStart(testSuite);
    }
    

    protected void runFinished(final Test testSuite) {
        SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    writeProtocolFooter(System.currentTimeMillis() - startTime);
                }
            });
        
        super.runFinished(testSuite);
    }
    

    public void testFailed(final int status, final Test test, final Throwable t) {
        super.testFailed(status, test, t);
        
        SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    fProtocolView.addFailure(test, t);
                }
            }
                                   );
    }

    
    protected void reset() {
        super.reset();
        fProtocolView.clear();
    }

    protected JPanel createFailedPanel() {
        JPanel failedPanel= super.createFailedPanel();

        // protocol button
        fProtocolButton = new JButton("Protocol");
        fProtocolButton.setToolTipText("View test protocol");
        fProtocolButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    viewProtocol();
                }
            }
                                          );
     
        failedPanel.add(fProtocolButton);
     
        return failedPanel;
    }
   
    protected TestResult createTestResult() {
        fLoggedTestResult = new LoggedTestResult();
        return fLoggedTestResult;
    }
 
    protected JFrame createUI(String suiteName) {
        JFrame frame = super.createUI(suiteName);
        fProtocolView = new DefaultTestProtocolView(frame, "Test Protocol", false);
        
        return frame;
    }
    /**
     * View the test protocol.
     *
     */
    protected void viewProtocol() {
        fProtocolView.show();
    }




    /**
     * Adds a message to the test protocol.
     *
     * @param message message text
     */
    public void addMessage(String message) {
        fProtocolView.addMessage(message);
    }


    protected void writeProtocolHeader(Date start) {
        String lines = "-----------------------------------------------------------------------------------------------------------------------------";
        fProtocolView.addMessage(lines + "\n");
        fProtocolView.addMessage("Test Suite: " + getSuiteText() + "\n");

        fProtocolView.addMessage("Start time: " + dateFormat.format(start) + "\n");
        fProtocolView.addMessage(lines + "\n");
    }


    protected void writeProtocolFooter(long runTime) {
        String lines = "-----------------------------------------------------------------------------------------------------------------------------";
        fProtocolView.addMessage(lines + "\n");

        int count = fLoggedTestResult.runCount();
        int failed = fLoggedTestResult.failureCount();
        int errors = fLoggedTestResult.errorCount();

        fProtocolView.addMessage("Finished: " + elapsedTimeAsString(runTime) + " seconds\n");
        fProtocolView.addMessage("Tests total: " + count + "\n");
        fProtocolView.addMessage("Tests succeeded: " + (count - failed - errors) + "\n");
        fProtocolView.addMessage("Tests failed: " + failed + "\n");
        fProtocolView.addMessage("Errors: " + errors + "\n");
        fProtocolView.addMessage(lines + "\n");
    }

} 
